import React, { useState, useContext } from "react";
import { usePipelineMappings, useIgnoreResult } from "@qltr/queries";
import StoreContext from "@qltr/store";

import { useJobCore, jobStatus } from "@qltr/jobcore";
import useMediaQuery from "@mui/material/useMediaQuery";
import { useTheme } from "@mui/material/styles";
import Accordion from "@mui/material/Accordion";
import AccordionDetails from "@mui/material/AccordionDetails";
import AccordionSummary from "@mui/material/AccordionSummary";
import Typography from "@mui/material/Typography";

import Button from "@mui/material/Button";
import Dialog from "@mui/material/Dialog";
import DialogActions from "@mui/material/DialogActions";
import DialogContent from "@mui/material/DialogContent";
import DialogContentText from "@mui/material/DialogContentText";
import DialogTitle from "@mui/material/DialogTitle";

import IconButton from "@mui/material/IconButton";
import DeleteIcon from "@mui/icons-material/Delete";
import NotificationsIcon from "@mui/icons-material/Notifications";
import ReplayIcon from "@mui/icons-material/Replay";
import PhotoCameraIcon from "@mui/icons-material/PhotoCamera";

import CheckIcon from "@mui/icons-material/Check";
import ClearIcon from "@mui/icons-material/Clear";
import ViewColumnIcon from "@mui/icons-material/ViewColumn";
import PendingIcon from "@mui/icons-material/Pending";
import VisibilityIcon from "@mui/icons-material/Visibility";
import DoNotDisturbIcon from "@mui/icons-material/DoNotDisturb";

import Badge from "@mui/material/Badge";
import Stack from "@mui/material/Stack";
import Box from "@mui/material/Box";

import { asTree, asBranches, as1d } from "@qltr/util/pipelines.js";

const stopPropagation = (e) => e.stopPropagation() && e.preventDefault();

export default function FailingBox(props) {
  const { failingTest, alertClick } = props;
  const {
    class: testClass,
    name: testName,
    timestamp,
    silence,
    type,
    dailyFails,
    screenshot: screenshotUrl,
    console: cs,
    recentResults,
    message,
    isPipeline,
    job,
    pipeline,
  } = failingTest;
  const ignoreResult = useIgnoreResult();
  const runHistory = recentResults ? [...recentResults].reverse() : null;

  const { data: pipelineMappings, isLoading } = usePipelineMappings();
  const { jobCompose, jobIcon, pipelineIcon } = useJobCore();
  const { state: store, updateStore, removeFailure } = useContext(StoreContext);
  const theme = useTheme();
  const minifyActions = useMediaQuery(theme.breakpoints.down("sm"));
  const [open, setOpen] = useState(false);
  const toggleOpen = () => setOpen(!open);
  const [removeOpen, setRemoveOpen] = useState(false);
  const removeClick = () => setRemoveOpen(!removeOpen);

  const handleRemoveClose = (confirmed) => (e) => {
    stopPropagation(e);
    setRemoveOpen(false);
    if (!confirmed) return;
    ignoreResult(failingTest);
  };

  function badgeColor() {
    if (dailyFails === 1) return "primary";
    else if (dailyFails === 2) return "secondary";
    else if (dailyFails < 6) return "warning";
    return "error";
  }

  const retryPipelineTest = () => {
    const primaries = pipelineMappings.filter((m) =>
      m.find((t) => t.name === testName)
    );
    const builderCache = {
      branches: asBranches(primaries),
      tree: asTree(primaries),
      selectedBranches: as1d(primaries),
      isTriage: store.triageFailing,
    };
    const pipeline = jobCompose(builderCache);
    if (store.focusJob) toPipeline(pipeline.id);
  };

  const retryTest = () => {
    if (isPipeline) return retryPipelineTest();
    const jobId = jobCompose({
      testNames: [testName],
      isTriage: store.triageFailing,
    });
    if (store.focusJob) toJob(jobId);
  };

  const navigateToJob = () => {
    if (pipeline) return toPipeline(pipeline.id);
    toJob(job.jobId);
  };

  const jobOnClick = () => {
    if (pipeline) return navigateToJob();
    if (!job) return retryTest();
    navigateToJob();
  };

  function boxIcon() {
    if (pipeline) return pipelineIcon(pipeline);
    if (!job) return <ReplayIcon />;
    return jobIcon(job);
  }

  return (
    <Accordion
      expanded={open}
      onChange={toggleOpen}
      disableGutters={true}
      square
    >
      <AccordionSummary
        style={{
          backgroundColor: "rgba(0, 0, 0, .03)",
          flexWrap: "wrap",
        }}
      >
        <Badge
          sx={{ mr: 2 }}
          color={badgeColor()}
          badgeContent={dailyFails}
          anchorOrigin={{
            vertical: "top",
            horizontal: "left",
          }}
        ></Badge>
        <Badge
          sx={{ mr: 2 }}
          style={{ whiteSpace: "nowrap", left: "3.125rem" }}
          badgeContent={new Date(timestamp).toLocaleTimeString("en-US")}
          anchorOrigin={{
            vertical: "bottom",
            horizontal: "left",
          }}
        ></Badge>
        <Dialog
          open={removeOpen}
          onClose={handleRemoveClose()}
          sx={{ "& .MuiDialog-paper": { width: "80%", maxHeight: 435 } }}
          maxWidth="xs"
        >
          <DialogTitle>Remove failure?</DialogTitle>
          <DialogContent>
            <DialogContentText>
              This will remove 1 test from the database
            </DialogContentText>
          </DialogContent>
          <DialogActions>
            <Button onClick={handleRemoveClose()}>Cancel</Button>
            <Button onClick={handleRemoveClose(true)} autoFocus>
              Yes
            </Button>
          </DialogActions>
        </Dialog>

        <Typography component={"span"} style={{ wordBreak: "break-word" }}>
          {`${testClass}#`}
          <Box fontWeight="bold" display="inline">
            {testName}{" "}
          </Box>
          <br />
          <div>
            <span className="recent-results">
              {runHistory &&
                runHistory.map(
                  (v, i) =>
                    (!v.failed && (
                      <CheckIcon
                        key={i}
                        color="success"
                        titleAccess={v.timestamp}
                      />
                    )) || (
                      <ClearIcon
                        key={i}
                        color="error"
                        titleAccess={v.timestamp}
                      />
                    )
                )}
            </span>
            {isPipeline && <ViewColumnIcon />}
          </div>
        </Typography>

        <Stack
          onClick={stopPropagation}
          direction={minifyActions ? "column" : "row"}
          sx={{ ml: "auto", mb: "auto", mt: "auto", whiteSpace: "nowrap" }}
        >
          <a href={screenshotUrl}>
            <IconButton aria-label="photo" component="span">
              <PhotoCameraIcon />
            </IconButton>
          </a>

          <IconButton aria-label="retry" component="span" onClick={jobOnClick}>
            {boxIcon()}
          </IconButton>

          <IconButton
            aria-label="silence"
            component="span"
            color={silence && silence.expires ? "primary" : "default"}
            onClick={alertClick}
            onContextMenu={alertClick}
          >
            <NotificationsIcon titleAccess={silence && silence.expires} />
          </IconButton>
          <IconButton
            color="error"
            aria-label="delete"
            component="span"
            onClick={removeClick}
          >
            <DeleteIcon />
          </IconButton>
        </Stack>
      </AccordionSummary>
      <AccordionDetails>
        <Typography component={"span"} style={{ wordBreak: "break-word" }}>
          {message}
        </Typography>
      </AccordionDetails>
    </Accordion>
  );
}
